@@ -37,25 +37,23 @@
 ##' @param testMany This takes logical value `TRUE` or `FALSE`. If `TRUE`, the `testCov` will contain all the variables in `CovData` provided `testCov` is set to be `NULL`. The default value is `TRUE` which does not do anything if `testCov` is not `NULL`.
 ##' @param ctrlMany This takes logical value `TRUE` or `FALSE`. If `TRUE`, all variables except `testCov` are considered as control covariates provided `ctrlCov` is set to be `NULL`. The default value is `FALSE`.
 ##' @param nRef The number of randomly picked reference taxa used in phase 1. Default number is `40`.
-##' @param nPermu The number of permutation used in phase 1. Default number is `40`.
-##' @param x1permut This takes a logical value `TRUE` or `FALSE`. If true, it will permute the variables in testCov. If false, it will use residual-permutation proposed by Freedman and Lane (1983). Default is 'TRUE'.
 ##' @param refTaxa A vector of taxa or OTU or ASV names. These are reference taxa specified by the user to be used in phase 1. If the number of reference taxa is less than 'nRef', the algorithm will randomly pick extra reference taxa to make up 'nRef'. The default is `NULL` since the algorithm will pick reference taxa randomly.
-##' @param reguMethod regularization approach used in phase 1 of the algorithm. Default is `"mcp"`. Other methods are under development.
-##' @param fwerRate The family wise error rate for identifying taxa/OTU/ASV associated with `testCov` in phase 1. Default is `0.25`.
+##' @param adjust_method The adjusting method used for p value adjustment. Same as p.adjust function in R.
+##' @param fdrRate The false discovery rate for identifying taxa/OTU/ASV associated with `testCov`. Default is `0.25`.
 ##' @param sequentialRun This takes a logical value `TRUE` or `FALSE`. Default is `FALSE`. This argument could be useful for debug.
 ##' @param paraJobs If `sequentialRun` is `FALSE`, this specifies the number of parallel jobs that will be registered to run the algorithm. If specified as `NULL`, it will automatically detect the cores to decide the number of parallel jobs. Default is `NULL`. It is safe to have 4gb memory per job. It may be needed to reduce the number of jobs if memory is limited.
 ##' @param standardize This takes a logical value `TRUE` or `FALSE`. If `TRUE`, all design matrix X in phase 1 and phase 2 will be standardized in the analyses. Default is `FALSE`.
 ##' @param nRefMaxForEsti The maximum number of reference taxa used in phase 2. The default is `1`.
 ##' @param bootB Number of bootstrap samples for obtaining confidence interval of estimates in phase 2. The default is `500`.
-##' @param bootLassoAlpha The significance level in phase 2. Default is `0.05`.
 ##' @param refReadsThresh The threshold of non-zero sequencing reads for choosing the reference taxon in phase 2. The default is `0.2` which means at least 20% non-zero sequencing reads.
+##' @param taxkeepThresh The threshold of number of non-zero sequencing reads for each taxon to be included into the analysis.
 ##' @param SDThresh The threshold of standard deviations of sequencing reads for choosing the reference taxon in phase 2. The default is `0.5` which means the standard deviation of sequencing reads should be at least `0.5`.
 ##' @param SDquantilThresh Threshold for the quantile of standard deviation for selecting final reference taxon
 ##' @param balanceCut The threshold of non-zero sequencing reads in each group of a binary variable for choosing the reference taxon in phase 2. The default number is `0.2` which means at least 20% sequencing reads are non-zero in each group.
 ##' @param seed Random seed for reproducibility. Default is `1`.
 ##' @return A list containing the estimation results.
 ##'
-##' - `analysisResults$estByCovList`: A list containing estimating results for all the variables in `testCov`. See details.
+##' - `analysisResults$sig_list_each_mean`: A list containing estimating results for all the variables in `testCov`. See details.
 ##'
 ##' - `covariatesData`: A dataset containing covariates and confounders used in the analyses.
 ##'
@@ -72,9 +70,8 @@
 ##'                 linkIDname = "id",
 ##'                 testCov = c("v1", "v2"),
 ##'                 ctrlCov = c("v3"), nRef = 3,
-##'                 nPermu = 3,
 ##'                 paraJobs = 2,
-##'                 fwerRate = 0.25,
+##'                 fdrRate = 0.25,
 ##'                 bootB = 5)
 ##'
 ##'}
@@ -84,9 +81,6 @@
 ##' @references Li et al.(In press) IFAA: Robust association identification and Inference For Absolute Abundance in microbiome analyses. Journal of the American Statistical Association
 ##' @references Zhang CH (2010) Nearly unbiased variable selection under minimax concave penalty. Annals of Statistics. 38(2):894-942.
 ##' @references Freedman and Lane (1983) A non-stochastic interpretation of reported significance levels. Journal of Business & Economic Statistics. 1(4):292-298.
-
-##' @importFrom stats cor kmeans na.omit quantile sd
-##' @importFrom utils read.csv read.table tail
 ##' @importFrom methods as
 ##' @importFrom foreach `%dopar%` foreach
 ##' @importFrom future availableCores
@@ -95,10 +89,13 @@
 ##' @importFrom picasso picasso
 ##' @importFrom qlcMatrix corSparse
 ##' @import mathjaxr
+##' @import glmnet
+##' @import stats
+##' @import utils
 ##' @export
 ##' @md
 
-
+## arg deleted: nperm, x1permt, reguMethod, method, bootLassoAlpha
 
 
 IFAA=function(
@@ -110,18 +107,17 @@ IFAA=function(
   testMany=TRUE,
   ctrlMany=FALSE,
   nRef=40,
-  nRefMaxForEsti=1,
-  nPermu=40,
-  x1permut=TRUE,
+  nRefMaxForEsti=2,
   refTaxa=NULL,
-  reguMethod=c("mcp"),
-  fwerRate=0.25,
+  # refTaxa_P2=NULL,
+  adjust_method="BY",
+  fdrRate=0.25,
   paraJobs=NULL,
   bootB=500,
-  bootLassoAlpha=0.05,
   standardize=FALSE,
   sequentialRun=FALSE,
   refReadsThresh=0.2,
+  taxkeepThresh=0,
   SDThresh=0.05,
   SDquantilThresh=0,
   balanceCut=0.2,
@@ -133,7 +129,8 @@ IFAA=function(
   start.time = proc.time()[3]
   runMeta=metaData(MicrobData=MicrobData,CovData=CovData,
                    linkIDname=linkIDname,testCov=testCov,
-                   ctrlCov=ctrlCov,testMany=testMany,ctrlMany=ctrlMany)
+                   ctrlCov=ctrlCov,testMany=testMany,
+                   ctrlMany=ctrlMany,taxkeepThresh=taxkeepThresh)
   data=runMeta$data
   results$covariatesData=runMeta$covariatesData
   binaryInd=runMeta$binaryInd
@@ -150,30 +147,51 @@ IFAA=function(
 
   if(length(refTaxa)>0){
     if(sum(refTaxa%in%microbName)!=length(refTaxa)){
-      stop("Error: One or more of the specified reference taxa have no sequencing reads
+      stop("Error: One or more of the specified reference taxa in phase 1 have no sequencing reads
       or are not in the data set. Double check the names of the reference taxa and their
            sparsity levels.")
     }
   }
 
+  # if(length(refTaxa_P2)>0){
+  #   if(sum(refTaxa_P2%in%microbName)!=length(refTaxa_P2)){
+  #     stop("Error: One or more of the specified reference taxa in phase 2 have no sequencing reads
+  #     or are not in the data set. Double check the names of the reference taxa and their
+  #          sparsity levels.")
+  #   }
+  # }
+
+  if (nRefMaxForEsti<2) {
+    nRefMaxForEsti<-2
+    warning("Warning: Needs at least two final reference taxon for estimation.")
+  }
+
+  # if (length(refTaxa_P2)>0) {
+  #   nRef=length(refTaxa_P2)
+  # }
+
   if(nRef>(length(microbName))){
     stop("Error: number of random reference taxa can not be larger than the total number
            of taxa in the data. Try lower nRef")
   }
 
   refTaxa_newNam=newMicrobNames[microbName%in%refTaxa]
+  # refTaxa_P2_newNam=newMicrobNames[microbName%in%refTaxa_P2]
+
 
   results$analysisResults=Regulariz(data=data,testCovInd=testCovInd,
                                     testCovInOrder=testCovInOrder,
                                     testCovInNewNam=testCovInNewNam,
                                     microbName=microbName,nRef=nRef,
                                     nRefMaxForEsti=nRefMaxForEsti,
-                                    nPermu=nPermu,binaryInd=binaryInd,
-                                    x1permut=x1permut,
+                                    binaryInd=binaryInd,
                                     covsPrefix=covsPrefix,Mprefix=Mprefix,
-                                    refTaxa=refTaxa_newNam,paraJobs=paraJobs,
-                                    reguMethod=reguMethod,fwerRate=fwerRate,
-                                    bootB=bootB,bootLassoAlpha=bootLassoAlpha,
+                                    refTaxa=refTaxa_newNam,
+                                    # refTaxa_P2=refTaxa_P2_newNam,
+                                    paraJobs=paraJobs,
+                                    adjust_method=adjust_method,
+                                    fwerRate=fdrRate,
+                                    bootB=bootB,
                                     standardize=standardize,
                                     sequentialRun=sequentialRun,
                                     allFunc=allFunc,refReadsThresh=refReadsThresh,
@@ -187,15 +205,12 @@ IFAA=function(
   results$ctrlCov=ctrlCov
   results$microbName=microbName
   results$bootB=bootB
-  results$bootLassoAlpha=bootLassoAlpha
   results$refReadsThresh=refReadsThresh
   results$balanceCut=balanceCut
   results$SDThresh=SDThresh
   results$paraJobs=paraJobs
   results$SDquantilThresh=SDquantilThresh
   results$nRef=nRef
-  results$nPermu=nPermu
-  results$x1permut=x1permut
 
   if(length(seed)==1){
     results$seed=seed
